/*
 * Main.java
 *
 * Created on 5. listopad 2006, 9:44
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package osmctverecku;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 *
 * @author pirko
 */
public class Main {
    
    // using of sets instead of lists may ensure that the states will not repeat
    private List<State> history = new ArrayList<State>();
    
    
    private Comparator badCount = new BadCountComparator();
    private Comparator manhattan = new ManhattanComparator();
    
    /** Creates a new instance of Main */
    public Main() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Main main = new Main();
        int sum = 0;
        for (int i=0;i<5;i++)
            sum += main.findSolution(new StateNode(State.genState(),null),main.badCount);
        
        double avg = ((double) sum) / 5.0;
        
        System.out.println("AVERAGE bad count: "+avg+"\n");
        
        sum = 0;
        for (int i=0;i<5;i++)
            sum += main.findSolution(new StateNode(State.genState(),null),main.manhattan);
        
        avg = ((double) sum) / 5.0;
        
        System.out.println("AVERAGE manhattan: "+avg);
        
    }
    
    class BadCountComparator implements Comparator {
        public int compare(Object state1, Object state2) {
            if (state1 == null) return Integer.MAX_VALUE;
            if (state2 == null) return Integer.MIN_VALUE;
            return ((StateNode)state1).getState().badCount() - ((StateNode)state2).getState().badCount();
        }
    }
    
    class ManhattanComparator implements Comparator {
        public int compare(Object state1, Object state2) {
            if (state1 == null) return Integer.MAX_VALUE;
            if (state2 == null) return Integer.MIN_VALUE;
            return ((StateNode)state1).getState().manhattan() - ((StateNode)state2).getState().manhattan();
        }
    }
    
    private int findSolution(StateNode start,Comparator<StateNode> comparator) {
        history.clear(); // ensure the empty history
        List<StateNode> queue = new ArrayList<StateNode>();
        queue.add(start);
        history.add(start.getState());
        
        int i = 0;
        
        System.out.println("STARTING WITH: ");
        start.getState().print();
        
        do {
            Collections.sort(queue,comparator);
            // get the first
            StateNode stateNode = queue.get(0);
            State state = stateNode.getState();
            // test if it is not solution
            if (state.isSolution()) {
                System.out.print("SOLUTION FOUND, ");
                int deep = stateNode.getDeep();
                System.out.println("DEEP: "+deep);
                return deep;
            }
            
            // if not expand child states to the queue
            State up = state.up();
            if (up !=null && !history.contains(up)) {
                queue.add(new StateNode(up,stateNode));
                history.add(up);
            }
            State down = state.down();
            if (down != null && !history.contains(down)) {
                queue.add(new StateNode(down,stateNode));
                history.add(down);
            }
            State left = state.left();
            if (left != null && !history.contains(left)) {
                queue.add(new StateNode(left,stateNode));
                history.add(left);
            }
            State right = state.right();
            if (right != null && !history.contains(right)){
                queue.add(new StateNode(right,stateNode));
                history.add(right);
            }
            
            
            // remove the first from the queue
            queue.remove(stateNode);
            
            
        } while (!queue.isEmpty() && i<300000);
        return i;
    }
    
}
